<?php
if (!defined('ABSPATH')) {
    exit;
}

function scv_add_student_meta_boxes() {
    add_meta_box(
        'student_details',
        'Student Details',
        'scv_student_meta_callback',
        'student_certificate',
        'normal',
        'high'
    );
}
add_action('add_meta_boxes', 'scv_add_student_meta_boxes');

function scv_student_meta_callback($post) {
    wp_nonce_field('scv_student_meta_save', 'scv_student_nonce');
    $fields = array(
        'gender' => 'Gender',
        'father_name' => 'Father Name',
        'course_name' => 'Course Name',
        'percentage' => 'Percentage',
        'year_of_passing' => 'Year of Passing',
        'academy_name' => 'Academy Name',
        'pdf_download_link' => 'PDF Download Link (URL)',
    );
    echo '<table class="form-table scv-student-table">';
    foreach ($fields as $key => $label) {
        $value = get_post_meta($post->ID, $key, true);
        if ($key === 'gender') {
            echo "<tr>";
            echo "<th><label for='$key'>$label:</label></th>";
            echo "<td><select id='$key' name='$key' class='regular-text scv-input scv-select'>";
            echo "<option value='Male'" . selected($value, 'Male', false) . ">Male</option>";
            echo "<option value='Female'" . selected($value, 'Female', false) . ">Female</option>";
            echo "</select></td>";
            echo "</tr>";
        } else {
            echo "<tr>";
            echo "<th><label for='$key'>$label:</label></th>";
            echo "<td><input type='text' id='$key' name='$key' value='" . esc_attr($value) . "' class='regular-text scv-input'></td>";
            echo "</tr>";
        }
    }
    $certificate_number = get_post_meta($post->ID, 'certificate_number', true);
    if ($certificate_number) {
        echo "<tr>";
        echo "<th><label>Certificate Number:</label></th>";
        echo "<td><input type='text' value='" . esc_attr($certificate_number) . "' class='regular-text scv-input' readonly></td>";
        echo "</tr>";
    }
    echo '</table>';
}

function scv_generate_unique_certificate_number($student_name) {
    $first_letter = strtoupper(substr(trim($student_name), 0, 1)) ?: 'X';
    $base_number = time() . sprintf("%03d", mt_rand(0, 999));
    $certificate_number = $first_letter . substr($base_number, -6);

    $args = array(
        'post_type' => 'student_certificate',
        'posts_per_page' => 1,
        'post_status' => array('publish', 'draft'),
        'meta_query' => array(
            array(
                'key' => 'certificate_number',
                'value' => $certificate_number,
                'compare' => '='
            )
        ),
        'fields' => 'ids'
    );
    $existing = get_posts($args);
    if (!empty($existing)) {
        $suffix = 1;
        do {
            $certificate_number = $first_letter . sprintf("%06d", (int)substr($base_number, -6) + $suffix);
            $existing = get_posts($args);
            $suffix++;
        } while (!empty($existing) && $suffix < 100);
    }
    return $certificate_number;
}

function scv_save_student_details($post_id) {
    if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) return;
    if (wp_is_post_revision($post_id)) return;
    if (get_post_type($post_id) !== 'student_certificate') return;
    if (!current_user_can('edit_post', $post_id)) return;

    if (empty($_POST) || !isset($_POST['scv_student_nonce']) || !wp_verify_nonce($_POST['scv_student_nonce'], 'scv_student_meta_save')) {
        return;
    }

    $fields = array(
        'gender' => 'sanitize_text_field',
        'father_name' => 'sanitize_text_field',
        'course_name' => 'sanitize_text_field',
        'percentage' => 'floatval',
        'year_of_passing' => 'intval',
        'academy_name' => 'sanitize_text_field',
        'pdf_download_link' => 'esc_url_raw',
    );

    $student_data = array();
    $has_data = false;

    $student_name = get_the_title($post_id);
    if (empty($student_name)) {
        add_settings_error('scv_messages', 'name_error', 'Student name is required.', 'error');
        return;
    }

    $existing_certificate = get_post_meta($post_id, 'certificate_number', true);
    if (empty($existing_certificate)) {
        $certificate_number = scv_generate_unique_certificate_number($student_name);
        update_post_meta($post_id, 'certificate_number', $certificate_number);
        $student_data['certificate_number'] = $certificate_number;
    } else {
        $student_data['certificate_number'] = $existing_certificate;
    }

    foreach ($fields as $field => $sanitize_function) {
        if (isset($_POST[$field])) {
            $value = call_user_func($sanitize_function, $_POST[$field]);
            if ($field === 'percentage' && ($value < 0 || $value > 100)) {
                $value = '';
                add_settings_error('scv_messages', 'percentage_error', 'Percentage must be between 0 and 100.', 'error');
            }
            if ($field === 'year_of_passing' && ($value < 1900 || $value > date('Y'))) {
                $value = '';
                add_settings_error('scv_messages', 'year_error', 'Year must be between 1900 and ' . date('Y') . '.', 'error');
            }
            if ($field === 'gender' && !in_array($value, array('Male', 'Female'))) {
                $value = '';
                add_settings_error('scv_messages', 'gender_error', 'Invalid gender selection.', 'error');
            }
            update_post_meta($post_id, $field, $value);
            $student_data[$field] = $value;
            if (!empty($value)) $has_data = true;
        }
    }

    if ($has_data && (isset($_POST['post_status']) && in_array($_POST['post_status'], array('publish', 'draft')))) {
        if (!class_exists('FPDF')) {
            add_settings_error('scv_messages', 'fpdf_error', 'FPDF library is missing.', 'error');
            return;
        }

        $settings = scv_get_customization_settings(); // Ensure updated settings are used

        try {
            $pdf = new FPDF('L', 'mm', array(297, 210));
            $pdf->AddPage();

            $background_path = SCV_PLUGIN_DIR . 'images/' . $settings['background_template'];
            if (file_exists($background_path)) {
                $pdf->Image($background_path, 0, 0, 297, 210);
            } else {
                error_log('Background file not found: ' . $background_path);
            }

            $logo_path = $settings['logo_path'];
            if ($logo_path && file_exists($logo_path)) {
                $pdf->Image($logo_path, 10, 10, 50);
            }

            // Registration Prefix
            $pdf->SetFont($settings['font_choice'], 'B', 10);
            list($r, $g, $b) = sscanf($settings['academy_color'], "%02x%02x%02x") ?: array(0, 0, 0);
            $pdf->SetTextColor($r, $g, $b);
            $pdf->SetXY(10, 30);
            $pdf->Cell(0, 5, $settings['reg_prefix'] . $student_data['certificate_number'], 0, 1, 'C');

            // Academy Name
            $pdf->SetFont($settings['font_choice'], 'B', $settings['academy_size']);
            list($r, $g, $b) = sscanf($settings['academy_color'], "%02x%02x%02x") ?: array(0, 0, 0);
            $pdf->SetTextColor($r, $g, $b);
            $pdf->SetXY(10, 40);
            $pdf->Cell(0, 10, $settings['academy_name'], 0, 1, 'C');

            // Certification Line 1
            $pdf->SetFont($settings['font_choice'], '', 12);
            list($r, $g, $b) = sscanf($settings['certification_line1_color'], "%02x%02x%02x") ?: array(0, 0, 0);
            $pdf->SetTextColor($r, $g, $b);
            $pdf->SetXY(10, 55);
            $pdf->MultiCell(0, 5, $settings['certification_line1'], 0, 'C');

            // Certification Line 2
            list($r, $g, $b) = sscanf($settings['certification_line2_color'], "%02x%02x%02x") ?: array(0, 0, 0);
            $pdf->SetTextColor($r, $g, $b);
            $pdf->SetXY(10, 65);
            $pdf->MultiCell(0, 5, $settings['certification_line2'], 0, 'C');

            // Certificate Title
            $pdf->SetFont($settings['font_choice'], 'B', $settings['title_size']);
            list($r, $g, $b) = sscanf($settings['title_color'], "%02x%02x%02x") ?: array(0, 0, 0);
            $pdf->SetTextColor($r, $g, $b);
            $pdf->SetXY(10, 80);
            $pdf->Cell(0, 10, $settings['certificate_title'], 0, 1, 'C');

            // Certification Text
            $pdf->SetFont($settings['font_choice'], '', $settings['certification_size']);
            list($r, $g, $b) = sscanf($settings['certification_color'], "%02x%02x%02x") ?: array(0, 0, 0);
            $pdf->SetTextColor($r, $g, $b);
            $pdf->SetXY(10, 100);
            $text = $settings['certification_text'];
            $father_text = $student_data['father_name'] ? ($student_data['gender'] === 'Female' ? 'Daughter of ' : 'Son of ') . $student_data['father_name'] : '';
            $replacements = array(
                '[student_name]' => $student_name,
                '[father_name]' => $father_text,
                '[course_name]' => $student_data['course_name'] ?: 'N/A',
                '[year_of_passing]' => $student_data['year_of_passing'] ?: 'N/A',
                '[percentage]' => $student_data['percentage'] ? "{$student_data['percentage']}%" : 'N/A',
                '[academy_name]' => $student_data['academy_name'] ?: 'N/A',
            );
            $text = str_replace(array_keys($replacements), array_values($replacements), $text);
            $text .= "\nCertificate Number: " . $student_data['certificate_number'];
            $pdf->MultiCell(0, 10, $text, 0, 'C');

            // Additional Info
            $pdf->SetFont($settings['font_choice'], 'I', 10);
            list($r, $g, $b) = sscanf($settings['additional_info_color'], "%02x%02x%02x") ?: array(0, 0, 0);
            $pdf->SetTextColor($r, $g, $b);
            $pdf->SetXY(10, 180);
            $pdf->Cell(0, 5, $settings['additional_info'], 0, 1, 'C');

            $signature_path = $settings['signature_path'];
            if ($signature_path && file_exists($signature_path)) {
                $pdf->Image($signature_path, 10, 150, 40, 20);
            }

            $seal_path = $settings['seal_path'];
            if ($seal_path && file_exists($seal_path)) {
                list($seal_width, $seal_height) = getimagesize($seal_path);
                $pdf->Image($seal_path, 247, 150, 30, 30 * $seal_height / $seal_width);
            }

            $upload_dir = wp_upload_dir();
            $pdf_dir = $upload_dir['path'];
            $pdf_file = $pdf_dir . "/certificate-$post_id.pdf";

            if (!file_exists($pdf_dir)) wp_mkdir_p($pdf_dir);
            $pdf->Output('F', $pdf_file, true);
            if (file_exists($pdf_file)) {
                chmod($pdf_file, 0644);
                $pdf_url = $upload_dir['url'] . "/certificate-$post_id.pdf";
                update_post_meta($post_id, 'pdf_download_link', $pdf_url);
                add_settings_error('scv_messages', 'pdf_success', 'Certificate generated! Number: ' . $student_data['certificate_number'], 'success');
            }
        } catch (Exception $e) {
            add_settings_error('scv_messages', 'pdf_error', 'PDF generation failed: ' . $e->getMessage(), 'error');
        }
    }
}
add_action('save_post', 'scv_save_student_details', 10, 1);

function scv_admin_notices() {
    settings_errors('scv_messages');
}
add_action('admin_notices', 'scv_admin_notices');
?>